@@ -8,7 +8,7 @@ import pytest
 import warnings
 import threading
 from time import sleep
-from multiprocessing import util, current_process
+from multiprocessing import util, current_process, Manager
 from pickle import PicklingError, UnpicklingError
 
 import cloudpickle
@@ -375,11 +375,7 @@ class TestExecutorDeadLock(ReusableExecutorMixin):
         """Test the race conditions in reusable_executor crash handling"""
         pytest.importorskip("psutil")  # required for kill_friend & co
 
-        if (
-            sys.platform == "win32"
-            and sys.version_info >= (3, 8)
-            and n_proc > 5
-        ):
+        if sys.platform == "win32" and n_proc > 5:
             pytest.skip(
                 "On win32, the paging size can be too small to import numpy "
                 "multiple times in the sub-processes (imported when loading "
@@ -949,6 +945,133 @@ class TestGetReusableExecutor(ReusableExecutorMixin):
         executor4 = get_reusable_executor()
         assert executor4 is executor3
 
+    def test_no_crash_on_stdin_access_in_child_subprocess_run(self):
+        # non-regression test for #420.
+        def call_subprocess():
+            sleep(0.01)
+            subprocess.run(
+                [sys.executable, "-c", "import sys; sys.stdin.read(0)"]
+            ).check_returncode()
+
+        executor = get_reusable_executor(max_workers=4, timeout=2)
+        executor.submit(call_subprocess).result()
+
+    def test_faulthandler_enabled(self):
+        cmd = """if 1:
+            from loky import get_reusable_executor
+            import faulthandler
+
+            def f(i):
+                if {expect_enabled}:
+                    assert faulthandler.is_enabled()
+                else:
+                    assert not faulthandler.is_enabled()
+                if i == 5:
+                    faulthandler._sigsegv()
+
+            if {enable_faulthandler_via_initializer}:
+                executor = get_reusable_executor(max_workers=2, initializer=faulthandler.enable)
+            else:
+                executor = get_reusable_executor(max_workers=2)
+
+            list(executor.map(f, range(10)))
+            # This should always trigger a crash.
+        """
+
+        def check_faulthandler_output(
+            expect_enabled=True, enable_faulthandler_via_initializer=False
+        ):
+            p = subprocess.Popen(
+                [
+                    sys.executable,
+                    "-c",
+                    cmd.format(
+                        expect_enabled=expect_enabled,
+                        enable_faulthandler_via_initializer=enable_faulthandler_via_initializer,
+                    ),
+                ],
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+            )
+            p.wait()
+            out, err = p.communicate()
+
+            # The worker is always expected to crash, irrespective of whether
+            # faulthandler is enabled or not.
+            assert p.returncode == 1, out.decode()
+
+            if expect_enabled:
+                assert b"Current thread" in err, err.decode()
+
+        original_pythonfaulthandler_env = os.environ.pop(
+            "PYTHONFAULTHANDLER", None
+        )
+        try:
+            # Case 1: faulthandler should be automatically enabled by default.
+            check_faulthandler_output(expect_enabled=True)
+
+            # Case 2: faulthandler should also be enabled when
+            # PYTHONFAULTHANDLER=1 is set.
+            os.environ["PYTHONFAULTHANDLER"] = "1"
+            check_faulthandler_output(expect_enabled=True)
+
+            # Case 3: faulthandler should not be enabled when
+            # PYTHONFAULTHANDLER=0 is set explicitly.
+            os.environ["PYTHONFAULTHANDLER"] = "0"
+            check_faulthandler_output(expect_enabled=False)
+
+            # Case 4: faulthandler can also be enabled manually via the initializer.
+            del os.environ["PYTHONFAULTHANDLER"]
+            check_faulthandler_output(
+                expect_enabled=True, enable_faulthandler_via_initializer=True
+            )
+        finally:
+            if original_pythonfaulthandler_env is None:
+                os.environ.pop("PYTHONFAULTHANDLER", None)
+            else:
+                os.environ["PYTHONFAULTHANDLER"] = (
+                    original_pythonfaulthandler_env
+                )
+
+    @pytest.mark.parametrize("n_cpus", [1, 2, 4])
+    def test_reusable_executor_queue_size(self, n_cpus):
+        # Check that the reusable executor can be lauched with more workers
+        # than the number of CPUs with no limitation on the queue_size.
+        #
+        # Non-regression test for https://github.com/joblib/loky/issues/396.
+
+        get_reusable_executor().shutdown()
+
+        import os
+
+        original_n_cpus = os.environ.get("LOKY_MAX_CPU_COUNT", None)
+        try:
+            os.environ["LOKY_MAX_CPU_COUNT"] = f"{n_cpus}"
+
+            n_workers = 2 * n_cpus + 2
+
+            manager = Manager()
+            barrier = manager.Barrier(n_workers, timeout=1)
+
+            def check_running(i, barrier):
+                print(f"TASK {i} running")
+                barrier.wait()
+                return True
+
+            executor = get_reusable_executor(max_workers=n_workers)
+            futures = [
+                executor.submit(check_running, i, barrier)
+                for i in range(n_workers)
+            ]
+            assert all(r.result() for r in futures)
+        finally:
+            manager.shutdown()
+            executor.shutdown()
+            if original_n_cpus is None:
+                os.environ.pop("LOKY_MAX_CPU_COUNT", None)
+            else:
+                os.environ["LOKY_MAX_CPU_COUNT"] = original_n_cpus
+
 
 class TestExecutorInitializer(ReusableExecutorMixin):
     def _initializer(self, x):
@@ -1052,7 +1175,7 @@ def test_no_crash_max_workers_on_windows():
         assert executor.submit(lambda: None).result() is None
 
     # No warning on any OS when max_workers does not exceed the limit.
-    assert len(record) == 0
+    assert len(record) == 0, [w.message for w in record]
     assert before_downsizing_executor is executor
     assert len(executor._processes) == _MAX_WINDOWS_WORKERS
 
